'// A simple inventory example
#include once "fbList.bi"


type t_Item
	Name as string
	Amount as integer
end type
type t_Inventory extends t_List
	declare function Pickup(byref itemName as string, byval amount as integer = 1) as integer
	declare sub Drop(byref itemName as string, byval amount as integer = 1)
	declare function Show() as string
	
	declare operator [](byval index as integer) as t_Item ptr
end type
operator t_Inventory.[](byval index as integer) as t_Item ptr
	return cptr(t_Item ptr, this.GetPtr(index))
end operator
sub t_Inventory.Drop(byref itemName as string, byval amount as integer = 1)
	if amount <= 0 then '// We can't drop nothing!
		exit sub
	end if
	
	if itemName <> "Gold" and amount > 1 then '// No stacking of things other than gold!
		for g as integer = 1 to amount
			this.Drop(itemName, 1)
		next
	else
		dim as integer index
		dim as t_Item ptr newItem = new t_Item
		newItem->Name	=	itemName
		newItem->Amount	=	amount
		index = this.Find(newItem)
		if index > -1 then
			this[index]->Amount -= amount
			if this[index]->Amount <= 0 then
				this.Remove(index)
			end if
			if amount > 1 then
				print "Dropped "& amount &" "& itemName
			else
				print "Dropped "& itemName
			end if
		end if
	end if
end sub
function t_Inventory.Pickup(byref itemName as string, byval amount as integer = 1) as integer
	dim as integer index
	if amount <= 0 then '// We can't pick up nothing!
		return -1
	end if
	if itemName <> "Gold" and amount > 1 then '// No stacking of things other than gold!
		for g as integer = 1 to amount
			index = this.Pickup(itemName, 1)
		next
	else
		dim as t_Item ptr newItem = new t_Item
		newItem->Name	=	itemName
		newItem->Amount	=	amount
		if itemName = "Gold" then '// We can stack gold
			index = this.Find(newItem)
			if index > -1 then
				this[index]->Amount += amount
			else
				index = this.Insert(newItem)
			end if
		else
			index = this.Insert(newItem)
		end if
		if index = -1 then '// Something went wrong, delete it to avoid memory leak
			delete newItem
		else
			if amount > 1 then
				print "Picked up "& amount &" "& itemName
			else
				print "Picked up "& itemName
			end if
		end if
	end if
	return index
end function
function t_Inventory.Show() as string
	dim as string r
	this.ForEach(g as integer)
		if this[g]->Amount > 1 then
			r &= "	["& g &"] "& this[g]->Amount &" "& this[g]->Name & chr(13,10)
		else
			r &= "	["& g &"] "& this[g]->Name & chr(13,10)
		end if
	next
	if r = "" then
		r = "	<Empty"
	end if
	return r
end function
sub ItemCleanup(byref item as t_Item ptr)
	delete item
end sub
function ItemCompare(byval a as t_Item ptr, byval b as t_Item ptr) as integer
	if a->Name < b->Name then
		return -1
	elseif a->Name > b->Name then
		return 1
	end if
	return 0
end function

dim as t_Inventory Inventory

Inventory.CustomDeleteSub(@ItemCleanup)
Inventory.CustomCompareFunction(@ItemCompare)
Inventory.EnableSort()

Inventory.Pickup("Gold", 100)
Inventory.Pickup("Sword")
Inventory.Pickup("Gold", 100)
Inventory.Pickup("Gold", 14)
Inventory.Pickup("Shiny boots", 2)

print
print "Inventory:"
print Inventory.Show()

Inventory.Drop("Gold", 115)
Inventory.Drop("Shiny boots")

print
print "Inventory:"
print Inventory.Show()


print
print "Press any key to complete your adventure!"
sleep

end